# こちらは関数仕様書のサンプルです。
# Made by TechMatrix Corp. 2025/05/21
#

GRAPH_OPTION = 'Cluster=on'
"""グラフのオプション"""

GRAPH_VARIANT = 'Classic'
"""グラフの類別"""

import understand
import sys
import os
import logging

def GetFunctionData(func, md, rootFolder):
    """
    １関数のデータを取得します。
    """
    # 名前
    name = func.name()
    
    # 引数
    parameter = func.parameters(True)

    # 戻り値
    rettype = func.type()
    if not rettype:
        rettype = ''

    definedin = func.ref("definedin")
    if definedin is None:
        definedinName = ''
    else:
        definedinName = definedin.ent().longname()

    # ローカル変数
    loalVariables = func.ents("", "Local Object")
    localVariable = ''
    for val in loalVariables:
        localVariable += val.type() + ' ' + val.name() + '\n'
    
    # コメント
    # 上部のものと関数定義横のものがありますので、改行で繋げます。
    comment = ''
    comment = func.comments('before')
    comment += '\n'
    comment += func.comments('after')

    # アノテーション
    # 複数ある場合もあるので、改行で繋げます。
    annotate = ''
    annotations = func.annotations()
    for ano in annotations:
        annotate += ano.text()
        annotate += '\n'

    # マークダウン形式でファイルに出力します。
    md.write('### ' + name + '\n')
    md.write('***引数***\n' + parameter + '\n')
    md.write('***戻り値***\n' + rettype + '\n')
    md.write('***ローカル変数***\n' + localVariable)
    md.write('***定義ファイル***\n' + definedinName + '\n')
    md.write('***コメント**\n' + comment + '\n')
    md.write('***アノテーション***\n' + annotate + '\n')
    

    # コントロールフロー図をDotファイルに出力します。
    dotFilePath = os.path.join(rootFolder, func.longname() + ".dot")
    
    try:
        func.draw("Control Flow", dotFilePath, GRAPH_OPTION, GRAPH_VARIANT)
    except Exception as exp:
        logging.warning("'%s'のグラフ作成に失敗しました。: %s", func.name(), exp)
        return

    # 出力したDotファイルを開き、マークダウンで読み取れるように追加します。
    dotfile = open(dotFilePath, 'r')
    if not dotfile:
      return
    
    fd = dotfile.read()
    md.write('***処理フロー図***\n')
    md.write('```dot\n')
    md.write(fd + '\n')
    md.write('```\n')
    dotfile.close()

    # Dotファイルは不要なので削除します。
    os.remove(dotFilePath)
    md.write('---' + '\n\n')


# ログ出力レベルを設定
logging.basicConfig(format='%(levelname)s:%(message)s', level=logging.INFO)

if __name__ == '__main__':
    
    # Understandデータベースを開きます
    args = sys.argv
    db = understand.open(args[1])

    # 第３引数にフォルダ名が無い場合はoutputを指定します。
    if len(args) < 3:
        rootFolder = 'output'
    else:
        rootFolder = args[2]
    
    os.makedirs(rootFolder, exist_ok=True)
    mdFile = os.path.join(rootFolder, 'output.md')
    # 出力するファイルのインスタンス(md)を作成します。
    md = open(mdFile, mode='w', encoding='utf-8')

    logging.info("ドキュメント生成を開始します。フォルダ:%s", str(rootFolder))

    # 関数をフェッチして各関数に対しての説明文を作成します。
    md.write('# Functions' + '\n')
    funcs = db.ents("Function ~unknown ~unresolved")
    for func in funcs:
        GetFunctionData(func, md, rootFolder)
    md.write('***' + '\n\n')
    md.close()

    logging.info("ドキュメント生成が完了しました。")